%{
/* ----------------------------------------------------------------------------
 *  Copyright (C) 1995-2002 by Karim Kaschani
 *
 *  This file is part of the function calculation package FuncCalc.
 *
 *  FuncCalc is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  FuncCalc is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * ------------------------------------------------------------------------- */

#include "globals.h"
#include "y.tab.h"
#include "symtab.h"
#include <math.h>

#undef input	/* disable lex input() */
#undef unput	/* disable lex unput() */

extern complex	xArg, yArg, gArg;
extern char	*formula;
extern int	yyPos;





/* ----------------------------------------------------------------------------
 * input - substitute for lex input()
 * ------------------------------------------------------------------------- */

char input(void)
{
	yyPos++;
	return(*formula++);
}





/* ----------------------------------------------------------------------------
 * unput - substitute for lex unput()
 * ------------------------------------------------------------------------- */

void unput(char c)
{
	yyPos--;
	*--formula = c;
}

%}

%%

exp		return EXP;
ln		return LN;
log		return LOG;
sqrt		return SQRT;
sin		return SIN;
cos		return COS;
tan		return TAN;
cot		return COT;
sinh		return SINH;
cosh		return COSH;
tanh		return TANH;
coth		return COTH;
arcsin		return ARCSIN;
arccos		return ARCCOS;
arctan		return ARCTAN;
arccot		return ARCCOT;
arsinh		return ARSINH;
arcosh		return ARCOSH;
artanh		return ARTANH;
arcoth		return ARCOTH;
erf		return ERF;
erfc		return ERFC;
fact		return FACT;
sgn		return SGN;
int		return INT;
frac		return FRAC;
round		return ROUND;
re!		return SRE;
RE!		return SRE;
im!		return SIM;
IM!		return SIM;
abs		return ABS;
phi		return PHI;
re		return RE;
im		return IM;

j(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?))e {
		yylval.cval.re = 0;
		yylval.cval.im = atof(&yytext[1])*M_E;
		return NUMBER;
		}
j(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?))pi {
		yylval.cval.re = 0;
		yylval.cval.im = atof(&yytext[1])*M_PI;
		return NUMBER;
		}
j(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?))x {
		yylval.cval.re = -atof(&yytext[1])*xArg.im;
		yylval.cval.im = atof(&yytext[1])*xArg.re;
		return NUMBER;
		}
jx		{
		yylval.cval.re = -xArg.im;
		yylval.cval.im = xArg.re;
		return NUMBER;
		}
j(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?))y {
		yylval.cval.re = -atof(&yytext[1])*yArg.im;
		yylval.cval.im = atof(&yytext[1])*yArg.re;
		return NUMBER;
		}
jy		{
		yylval.cval.re = -yArg.im;
		yylval.cval.im = yArg.re;
		return NUMBER;
		}
j(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?)) {
		yylval.cval.re = 0;
		yylval.cval.im = atof(&yytext[1]);
		return NUMBER;
		}
(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?))e {
		yylval.cval.re = atof(yytext)*M_E;
		yylval.cval.im = 0;
		return NUMBER;
		}
(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?))pi {
		yylval.cval.re = atof(yytext)*M_PI;
		yylval.cval.im = 0;
		return NUMBER;
		}
(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?))x {
		yylval.cval.re = atof(yytext)*xArg.re;
		yylval.cval.im = atof(yytext)*xArg.im;
		return NUMBER;
		}
(([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?))y {
		yylval.cval.re = atof(yytext)*yArg.re;
		yylval.cval.im = atof(yytext)*yArg.im;
		return NUMBER;
		}
e		{
		yylval.cval.re = M_E;
		yylval.cval.im = 0;
		return NUMBER;
		}
j		{
		yylval.cval.re = 0;
		yylval.cval.im = 1;
		return NUMBER;
		}
pi		{
		yylval.cval.re = M_PI;
		yylval.cval.im = 0;
		return NUMBER;
		}
x		{
		yylval.cval.re = xArg.re;
		yylval.cval.im = xArg.im;
		return NUMBER;
		}
y		{
		yylval.cval.re = yArg.re;
		yylval.cval.im = yArg.im;
		return NUMBER;
		}
([0-9]+)|(([0-9]+(\.[0-9]+)?)([eE][-+]?[0-9]+)?) {
		yylval.cval.re = atof(yytext);
		yylval.cval.im = 0;
		return NUMBER;
		}
[A-Za-z][A-Za-z0-9]* {
		yylval.symp = lookSymtab(yytext);
		return NAME;
		}
[ \t]		;			/* ignore whitespace */
\n		return 0;		/* logical EOF */
.		return yytext[0];	/* any other character */
%%
